float4x4 wvp : WorldViewProjection;
float4x4 world : World;
float4x4 worldI : WorldI;
float4x4 viewI : ViewI;

struct appdata {
    float4 Position	 : POSITION;
    float4 texcoord0 : TEXCOORD0;
    float4 Normal	   : NORMAL;
};

struct vertexOutput {
    float4 Position  : POSITION;
    float4 texcoord0 : TEXCOORD0;
    float4 texcoord1 : TEXCOORD1;
    float4 texcoord2 : TEXCOORD2;
    float4 texcoord3 : TEXCOORD3;
    float4 texcoord4 : TEXCOORD4;
};

struct pixelOutput {
  float4 col : COLOR;
};

vertexOutput mainVS(appdata IN,
    uniform float4x4 WorldViewProj,
    uniform float4x4 World,
    uniform float4x4 WorldI,
    uniform float4x4 ViewI
) 
{
    vertexOutput OUT;

    OUT.Position = mul(WorldViewProj, IN.Position);
    OUT.texcoord0 = IN.texcoord0;

    OUT.texcoord1.xyz = mul(ViewI, float4(0.0, 0.0, 0.0, 1.0)).xyz;
    OUT.texcoord2.xyz = mul(World, IN.Position).xyz;
    OUT.texcoord3.xyz = mul(IN.Normal, WorldI).xyz;

    return OUT;
}

pixelOutput glass( vertexOutput IN, uniform samplerCUBE EnvironmentMap,  
                      uniform float minRefl, uniform float maxRefl, uniform float fresnelEXP, 
                      uniform float3 color, uniform float n)
{
    pixelOutput OUT; 

    float3 normal        = normalize(IN.texcoord3.xyz);
    float3 viewVector    = normalize(IN.texcoord2.xyz - IN.texcoord1.xyz);

    float3 reflVect = normalize(reflect(viewVector, normal));
    float4 reflColor = texCUBE(EnvironmentMap, reflVect);

    // compute fresnel equation
    float F = minRefl + (maxRefl-minRefl)*pow(1.0 - dot(-viewVector, reflVect), fresnelEXP);


    OUT.col = float4(F*reflColor.xyz, (1.0-F));

    return OUT;
}

float4 Color = {0.0f, 0.0f, 0.0f, 1.0f};

float Min
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 1.0;
	float uistep = 0.01;
> = 0.0;

float Max
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 1.0;
	float uistep = 0.01;
> = 1.0;

float FresnelExponent
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 100.0;
	float uistep = 1.00;
> = 1.0;


float N
<
	string gui = "slider";
	float uimin = 0.0;
	float uimax = 5.0;
	float uistep = 0.05;
> = 1.5;

texture cubeMap : EnvMap
<
    string File = "nvlobby_new.dds";
    string TextureType = "Cube";
>;

samplerCUBE environmentMapSampler = sampler_state
{
	Texture = <cubeMap>;

	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;

      AddressU  = Clamp;
      AddressV  = Clamp;
      AddressW  = Clamp;
};

technique Glass
{
	pass p0 
	{
    VertexShader = compile vs_2_0 mainVS(wvp, world, worldI, viewI);

		ZEnable      = true;
		ZWriteEnable = true;
    CullMode     = None;

    AlphaBlendEnable = true;
		SrcBlend = SrcColor;
		DestBlend = InvSrcColor;

		CullMode = CW;

    PixelShader = compile ps_2_0 glass(environmentMapSampler, Min, Max, FresnelExponent, Color, N);
	}
}


