struct VS_INPUT
{
	float4 position   : POSITION;        // object space position
        float4 normal     : NORMAL;           
};

struct VS_OUTPUT
{
   	float4 position   : POSITION;      // homogeneous clip space position  
   	float4 texcoord0  : TEXCOORD0;     // texture coordinate set 0  
};

struct FRAGOUT
{
	float4 col        : COLOR;
};

float4x4 wvp    : WorldViewProjection;
float4x4 worldI  : WorldI;

VS_OUTPUT skybox_vs(VS_INPUT IN, uniform float4x4 WorldViewProj, uniform float4x4 WorldI) 
{
    VS_OUTPUT OUT;

    OUT.position      = mul(WorldViewProj, IN.position);
    OUT.texcoord0.xyz = mul(IN.normal, WorldI).xyz;

    return OUT;
}

FRAGOUT skybox_fs(VS_OUTPUT IN, uniform samplerCUBE EnvironmentMap)
{
    FRAGOUT OUT; 

    float4 cubeColor  = texCUBE(EnvironmentMap, IN.texcoord0.xyz); 

    OUT.col = cubeColor;

    return OUT;
}

texture cubeMap : EnvMap
<
    string File = "nvlobby_new.dds";
    string TextureType = "Cube";
>;

samplerCUBE environmentMapSampler = sampler_state
{
	Texture = <cubeMap>;

	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;

      AddressU  = Clamp;
      AddressV  = Clamp;
      AddressW  = Clamp;
};

technique skybox
{
	pass p0 
	{		
                VertexShader = compile vs_1_1 skybox_vs(wvp, worldI);

		ZEnable = true;
		ZWriteEnable = true;
		CullMode = CCW;
                CullMode = None;
                ShadeMode = Gouraud; // Flat; Gouraud

                PixelShader = compile ps_1_1 skybox_fs(environmentMapSampler);
	}
}
