// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once

#include "nvsgcommon.h"

#include <GL/gl.h>  // standard OpenGL header satisfying OpenGL 1.1 API
#include <GL/glu.h> // OpenGL utility

// Calling convention define required below.
// Note: GLAPI is already defined in gl.h from Mesa 3-D graphics library (Linux build),
// where it is used to specify the linkage and not the calling convention.
#ifdef _WIN32
# define GLAPI __stdcall
#else
# define GLAPI 
#endif

// Note: Be sure to define GL_GLEXT_LEGACY when building on Linux with Mesa 3-D headers!
#include "GL/glext.h"   // OpenGL extension
#if defined(_WIN32)
# include "GL/wglext.h" // Microsoft specific OpenGL extensions
#elif defined(LINUX)
# include "GL/glx.h"
# include "nvgl/glxtypes.h"
#endif               
 
// STL header
#include <map>

// GL error tracing macro
#ifdef _DEBUG
# define ASSERT_GL_NO_ERROR               __ASSERT( ( glError = glGetError() ) == GL_NO_ERROR )
# define ASSERT_GL_NO_ERROR_IN_BEGIN_END  __ASSERT( glContext->m_inBeginEndLoop || ( glError = glGetError() ) == GL_NO_ERROR )
#else
# define ASSERT_GL_NO_ERROR
# define ASSERT_GL_NO_ERROR_IN_BEGIN_END
#endif

//! Namespace for OpenGL
/** This namespace wraps around the native OpenGL implementation.
  * It organizes caching, attribute handling, etc....
  * \note Do not use native OpenGL calls when extending NVSG or NVSG-based viewers.
  * Instead, use the implementation of the OpenGL commands contained in this namespace 
  * so you do not break caching, etc.
  *
  * \note Do not include other OpenGL headers. 
  * nvgl.h should be the only OpenGL header you need. */
namespace nvgl
{
  class GLContext;
  
  extern  NVSG_API GLContext                   * glContext;       //!<  pointer to the current GLContext
  extern  NVSG_API GLenum                        glError;         //!<  current OpenGL error code
#if defined(_WIN32)
  extern  NVSG_API std::map<HGLRC, GLContext*>   hglrcContextMap;
#elif defined(LINUX)
  extern  NVSG_API std::map<GLXContext, GLContext*> hglrcContextMap;  
#endif
  extern  bool                                   wglFunctionsInitialized; //!<  indicator of initialization state of wgl functions
}
