// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** @file */

#include "nvsgcommon.h"

#include "nvmath/Vec3f.h"
#include "nvmath/Vecnf.h"

namespace nvmath
{
  //! 4-component vector class.
  class Vec4f : public Vecnf<4>
  {
    public:
      //! Default constructor.
      /** For performance reasons no initialization is performed. */
      NVSG_API Vec4f( void );

      //! Constructor using four floats.
      /** A vector with the given values is constructed.  */
      NVSG_API Vec4f( float x   //!<  x component
                    , float y   //!<  y component
                    , float z   //!<  z component
                    , float w   //!<  w component
                    );

      //! Constructor using a 3-vector and a float
      NVSG_API Vec4f( const Vec3f &v3 //!<  vector of x, y, z component
                    , float w         //!<  w component
                    );
     
      //! Copy constructor.
      NVSG_API Vec4f( const Vecnf<4> &v //!<  vector to copy
                    );

      //! Set the vector with four float values.
      NVSG_API void set( float x    //!<  x component
                       , float y    //!<  y component
                       , float z    //!<  z component
                       , float w    //!<  w component
                       );
  };

  // - - - - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  // non-member functions
  // - - - - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  /*! \relates nvmath::Vec4f
   *  Multiplication of a scalar with a vector.
   *  \returns product of \a f with \a v */
  inline Vec4f operator*( float f         //!<  scalar to multiply with
                        , const Vec4f &v  //!<  vector to multiply
                        )
  {
    return( v * f );
  }

  // - - - - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  // inlines
  // - - - - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  inline Vec4f::Vec4f( void )
  {
  }

  inline Vec4f::Vec4f( float x, float y, float z, float w )
  {
    (*this)[0]=x;
    (*this)[1]=y;
    (*this)[2]=z;
    (*this)[3]=w;
  }

  inline Vec4f::Vec4f( const Vec3f &v3, float w )
  {
    (*this)[0] = v3[0];
    (*this)[1] = v3[1];
    (*this)[2] = v3[2];
    (*this)[3] = w;
  }

  inline Vec4f::Vec4f( const Vecnf<4> &v )
    : Vecnf<4>( v )
  {
  }

  inline void Vec4f::set( float x, float y, float z, float w )
  {
    (*this)[0]=x;
    (*this)[1]=y;
    (*this)[2]=z;
    (*this)[3]=w;
  }

} // end namspace nvmath
