// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include "nvsgcommon.h"

namespace nvsg
{
  //! Helper class for Quads and Triangles.
  /** A Face consists of indices into a vertex array, forming a face. */
  template<size_t n> struct Face
  {
    //! Random access operator for more convenient usage.
    /** \returns  index of vertex \a i of this Face in a vertex array */
    unsigned int& operator[]( size_t i    //!<  index into the Face<n> (0,1,...,n-1)
                            )
    {
      __ASSERT( i < n );
      return indices[i];
    }

    //! Random access operator for more convenient usage, constant version.
    /** \returns  index of vertex \a i of this Face in a vertex array */
    const unsigned int& operator[]( size_t i    //!<  index into the Face<n> (0,1,...,n-1)
                                  ) const
    {
      __ASSERT( i < n );
      return indices[i];
    }

    unsigned int indices[n];    //!<  Array of indices into a vertex array.
                                //!<  Indices have a precision of 32-bits!
  };

  typedef Face<3> Face3;    //!<  Shortcut for a 3-element face.
  typedef Face<4> Face4;    //!<  Shortcut for a 4-element face.
}
