// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
#include "ModelViewTraverser.h"

/** \file */

#include "nvsgcommon.h"

#include "ModelViewTraverser.h"
#include  "nvsg/ViewState.h"

namespace nvtraverser {

//! AppTraverser Class
/** This class provides the mechanism to apply changes and update the scene graph. 
  * (The 'apply' step of the application.) Use an AppTraverser derived class as a unique 
  * entry point for manipulating nodes in the scene, such as data from gadgets, 
  * animations, and interactions.
  * \note The current implementation takes care of the animation frames, camera clip planes
  * highlighting, etc...
  */
class AppTraverser : public ModelViewTraverser
{
public:
  //! Default constructor.
  NVSG_API AppTraverser();

protected:
  //!Default destructor.
  NVSG_API virtual ~AppTraverser();

  //! doApply override
  NVSG_API virtual void doApply( const nvsg::ViewState *viewState, const nvsg::Scene *scene );

  // handler routine overrides
  NVSG_API virtual void handleGeoNode( const nvsg::GeoNode *p );
  NVSG_API virtual void handleAnimatedTransform( const nvsg::AnimatedTransform *p );
  NVSG_API virtual void handleAnimatedQuads( const nvsg::AnimatedQuads *p );
  NVSG_API virtual void handleAnimatedTriangles( const nvsg::AnimatedTriangles *p );
  NVSG_API virtual void handleSkinnedTriangles( const nvsg::SkinnedTriangles *p );
  NVSG_API virtual void handleCgFx( const nvsg::CgFx *p );
  NVSG_API virtual void handleHighlightBox( const nvsg::HighlightBox *p, const nvsg::GeoSet * geoSet);

  // traverseLight override
  NVSG_API virtual void traverseLight( const nvsg::LightSource *p );

  // data accessible for derived classes
  bool   m_bApplyingCgFx;  //!< Flag that signals if a CgFX effect needs to be handled (tangentspace calculation).
  size_t m_animationFrame; //!< Current animation frame.
  
private:
  // Some data to cache so we can decide if we really have to apply the animation data. 
  // This does not help to prevent from unneeded traversal steps since we can't 
  // decide for derived classes. 
  const nvsg::ViewState * m_pLastViewState;
  const nvsg::Scene     * m_pLastScene;
  size_t                  m_lastAnimationFrame;
  bool                    m_bApplyAnimationKey;

private:
  // Calculate the bounding box of a geometry.
  NVSG_API void calcHighlightBox( 
    const nvsg::HighlightBox *p         // HighlightBox to hold box dimensions
  , const nvsg::GeoSet *geo             // geometry to be highlighted
  );

};

}
