// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include "nvsgcommon.h"

#include "nvsg/CullData.h"
#include "nvtraverser/CullTraverser.h"

namespace nvtraverser
{
  //! \c CullTraverser culling on frustum information.
  /** This \c CullTraverser gathers information on all \c Node s in the tree whether it's inside, outside or partially
    * within the current viewing frustum. All \c Node s (and potentially subtrees) that are completely outside are marked
    * as such and can be skipped on subsequent traversals. */
  class FrustumCullTraverser : public CullTraverser
  {
    public:
      //! Constructor
      NVSG_API FrustumCullTraverser(void);

    protected:
      //! Destructor
      NVSG_API virtual ~FrustumCullTraverser(void);
        
      // preTraverseRoot override
      NVSG_API virtual bool preTraverseRoot( const nvsg::SimpleCamera *p, const nvsg::Node *root );

      //  Reimplemented from CullTraverser
      NVSG_API virtual nvsg::CullCode cullCode( const nvsg::Drawable *p ) const;
      NVSG_API virtual nvsg::CullCode cullCode( const nvsg::Node *p ) const;

      NVSG_API virtual bool preTraverseTransform( const nvsg::Transform *p    //!< Transform node to traverse next 
                                                );
      NVSG_API virtual void postTraverseTransform( const nvsg::Transform *p   //!<  Transform node just traversed
                                                 );

    private:
      //  Determine the CullCode based on a sphere
      nvsg::CullCode _cullCode( const nvmath::Sphere3f &sphere ) const;

    private:
      const nvsg::SimpleCamera  * m_camera;
      mutable float               m_scaleFactor;
  };

}

