// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#pragma once
/** \file */

#include "nvsgcommon.h"

#include "nvtraverser/Traverser.h"

namespace nvtraverser
{
  //! Traverser that can smooth some drawables.
  /** Currently, this SmoothTraverser supports Triangles only. If these Triangles are non-trivially indexed, they are
    * changed to be trivially indexed.  */
  class SmoothTraverser : public Traverser
  {
    public:
      //! Constructor
      NVSG_API SmoothTraverser(void);

      //! Set the crease angle for smoothing.
      NVSG_API void  setCreaseAngle( float creaseAngle );

    protected:
      //! Protected destructor to prevent instantiation of a SmoothTraverser.
      NVSG_API virtual ~SmoothTraverser(void);

      //! Handle an AnimatedQuads object.
      /** Explicitly ignore these GeoSets on smoothing! */
      NVSG_API virtual void  handleAnimatedQuads( 
        const nvsg::AnimatedQuads *p      //!<  AnimatedQuads to handle
      );

      //! Handle an AnimatedTriangles object.
      /** Explicitly ignore these GeoSets on smoothing! */
      NVSG_API virtual void  handleAnimatedTriangles( 
        const nvsg::AnimatedTriangles *p  //!<  AnimatedTriangles to handle
      );

      //! Handle a GeoNode object.
      /** The GeoNode is the primary object to smooth.  */
      NVSG_API virtual void  handleGeoNode( 
        const nvsg::GeoNode *p            //!<  GeoNode to handle
      );

      //! Handle a Quads object.
      NVSG_API virtual void  handleQuads(
        const nvsg::Quads *p              //!<  Quads to handle
      );

      //! Handle a QuadStrips object.
      /** QuadStrips are changed to Quads here...  */
      NVSG_API virtual void  handleQuadStrips(
        const nvsg::QuadStrips *p         //!<  QuadStrips to handle
      );

      //! Handle an SkinnedTriangles object.
      /** Explicitly ignore these GeoSets on smoothing! */
      NVSG_API virtual void  handleSkinnedTriangles(
        const nvsg::SkinnedTriangles *p   //!<  SkinnedTriangles to handle
      );

      //! Handle a Triangles object.
      NVSG_API virtual void  handleTriangles(
        const nvsg::Triangles *p          //!<  Triangles to handle
      );

      //! Handle a TriStrips object.
      /** TriStrips are changed to Triangles here...  */
      NVSG_API virtual void  handleTriStrips(
        const nvsg::TriStrips *p          //!<  TriStrips to handle
      );

    private:
      float                                 m_creaseAngle;
      std::vector<const nvsg::GeoSet*>                       m_geoSets;
      std::vector<std::pair<const nvsg::QuadStrips*,const nvsg::Quads*> >     m_quadStripsQuads;
      std::vector<std::pair<const nvsg::TriStrips*,const nvsg::Triangles*> >  m_triStripsTriangles;
  };
}
