// Copyright NVIDIA Corporation 2002-2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES 

#include "nvsg/nvsgapi.h"

#include <qapplication.h>

#include "QtRAWnd.h"
#include "SimpleScene.h"

#include "nvsg/PerspectiveCamera.h"
#include "nvsg/PointLight.h"
#include "nvsg/Node.h"
#include "nvsg/Triangles.h"
#include "nvsg/Transform.h"
#include "nvsg/GeoNode.h"
#include "nvsg/PointLight.h"
#include "nvsg/Material.h"
#include "nvsg/FaceAttribute.h"
#include "nvsg/PerspectiveCamera.h"

#include "nvui/TrackballCamManip.h"

#include "nvmath/Vec3f.h"

// enable memory leak detection
#include "nvutil/DbgNew.h"

using namespace nvsg;
using namespace nvui;
using namespace nvmath;

int main( int argc, char ** argv )
{  
  // init global data required by the nvsg API
  nvsg::nvsgInitialize(true);

  QApplication::setColorSpec( QApplication::CustomColor );
  QApplication a( argc, argv );
  int retval = -1;

  if ( !QGLFormat::hasOpenGL() )
  {
	  qWarning( "This system has no OpenGL support. Exiting." );
	  return -1;
  }

  QtRAWnd * pRA = new QtRAWnd();
  assert(pRA);
  pRA->setCaption( "NVSG QT Example" );

  if(pRA->init())
  {    
    TrackballCamManip * pCamManip = new TrackballCamManip;

    const SimpleScene * pScene = SimpleScene::create();
    pScene->addRef();

    // Create camera
    PerspectiveCamera * pNCCamera = beginEdit( PerspectiveCamera::create() );

    // Make the (world space) window aspect ratio 
    // correspond to the (pixel space) viewport aspect ratio    
    pNCCamera->setName("ViewCamera");
    pNCCamera->setAspectRatio(pRA->getAspectRatio());   

    // Setup the cameras' headlight
    PointLight * pNCPointLight = beginEdit( PointLight::create() );
    pNCPointLight->setAmbientColor(Vec3f(1.f, 1.f, 1.f));
    pNCPointLight->setDiffuseColor(Vec3f(1.0f, 1.0f, 1.0f));
    pNCPointLight->setSpecularColor(Vec3f(1.0f, 1.0f, 1.0f));
    pNCPointLight->setAttenuation(1.0f, 0.0f, 0.0f);
    pNCPointLight->setPosition( Vec3f( 5.0f, 5.0f, 0.0f ) );
    pNCPointLight->setOrientation( Quatf( 0.0f, 0.0f, 0.0f, 1.0f ) );
    pNCCamera->addHeadLight( endEdit( pNCPointLight ) );    

    // Zoom all
    pNCCamera->zoom( pScene->getRootNode()->getBoundingSphere(), float(nvmath::PI_QUARTER));    

    pCamManip->attach(pNCCamera);
    
    pRA->setSceneData( pCamManip, pScene, NULL );   
         
    endEdit(pNCCamera);

    // setup done, let's go...

    pRA->show();    
    pRA->triggerRedraw();
    
    // do the application loop
    a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
    retval = a.exec();


    pScene->removeRef();
    // destroy the render area and clean up the not refcounted stuff
    pRA->destroy();
    delete pCamManip;
  }
  
  delete pRA;
  
  // free global data required by the nvsg API
  nvsg::nvsgTerminate();

  return retval;
}


