/***************************************************************************
 *   Copyright (C) 2004 by dsievers@NIVIDA                                 
 *                                                               
 ***************************************************************************/
#ifndef NVUIGLWXRAWND_H
#define NVUIGLWXRAWND_H

#include <sys/types.h>

#include <wx/glcanvas.h>

#include "nvsg/nvsgapi.h"
#include "nvgl/nvgl.h"
#include "nvui/RenderArea.h"


class GLwxRAWnd : public nvui::RenderArea, public wxGLCanvas
{

  //DECLARE_DYNAMIC_CLASS(GLwxRAWnd)

public:        
    GLwxRAWnd(wxWindow* parent, int* attribList, wxWindowID id=-1);
    ~GLwxRAWnd();
          
    enum ViewMode
    {
      VM_SET_ROTATION_POINT,
      VM_VIEWING
    };

    //! Set the size of the window.
    /** MFC window manager specific wrapper method.
      * \return true - everything went fine.*/
    virtual void setViewportSize( size_t width    //!< with in pixel
                                         , size_t height   //!< height in pixel
                                         );
      
    //! Create the render area.
    /** This method creates and initializes the render area.
      * \return true - everything went fine */      
    virtual bool init( RenderArea * shareArea = NULL  //!< Pointer to a render area to share display lists, textures, vertex programs, etc. with
                              );         

    //! wxWidgets interface
    virtual bool Destroy();

    //! RenderArea interface 
    virtual void destroy( void );
    
    void OnDraw(wxPaintEvent& event);     
    
    void OnSize(wxSizeEvent& event);

    void OnLButtonDown(wxMouseEvent& event);
    void OnLButtonUp(wxMouseEvent& event);
    void OnMButtonDown(wxMouseEvent& event);
    void OnMButtonUp(wxMouseEvent& event);
    void OnRButtonDown(wxMouseEvent& event);
    void OnRButtonUp(wxMouseEvent& event);
    void OnMouseMove(wxMouseEvent& event);
    void OnKeyDown(wxKeyEvent& event);

    void setViewMode(ViewMode mode);
      
    //! Trigger a redraw.  
    /** \note Use this method to trigger a redraw 
      * instead of calling the drawing method directly */
    virtual void triggerRedraw();
    
    //! Render the scene.
    /** Do the whole rendering of the scene. (app, cull, draw the whole tree) 
      * If you want to trigger this method, use TriggerRedraw() to force a redraw 
      * of the scene. */
    virtual void renderScene( void );  

    void makeCurrent();
                        
protected:

    GLXDrawable   m_drawable;
    GLXContext m_ctx;
    Display   *m_dpy;    
    ViewMode   m_viewMode;    

    DECLARE_EVENT_TABLE()
};


#endif
