/***************************************************************************
 *   Copyright (C) 2004 by dsievers@NVIDIA
 *   
 ***************************************************************************/
#include <wx/wx.h>


#include "nvsg/nvsgapi.h"
#include "nvutil/Assert.h"

#include "vwr.h"
#include "vwrview.h"
#include "vwrdocument.h"

#include "nvutil/Tools.h"
#include "nvsg/Scene.h"
#include "GLwxRAWnd.h"

// enable memory leak detection
#include "nvutil/DbgNew.h"

#define MENU_ID_SET_ROTATION_POINT 10007
#define MENU_ID_VIEW_ALL           10008

using namespace nvsg;

VwrApp* theApp=NULL;

IMPLEMENT_CLASS(VwrFrame, wxDocParentFrame)
BEGIN_EVENT_TABLE( VwrFrame, wxFrame )
  EVT_CLOSE( VwrFrame::OnCloseWindow )
	EVT_MENU( Menu_File_Quit, VwrFrame::OnQuit )
	EVT_MENU( Menu_File_About, VwrFrame::OnAbout )
  EVT_MENU( MENU_ID_SET_ROTATION_POINT, VwrFrame::OnSetRotationPoint )
  EVT_MENU( MENU_ID_VIEW_ALL, VwrFrame::OnViewAll )  
END_EVENT_TABLE()

IMPLEMENT_APP(VwrApp)

VwrApp::VwrApp()
{ 
}

VwrApp::~VwrApp()
{
}

bool VwrApp::OnInit()
{          
  theApp=this;

  // nvsg library one-time initialization
  nvsg::nvsgInitialize();

  // Create a document manager
  m_docManager = new wxDocManager;

  // Create a template relating drawing documents to their views
  (void) new wxDocTemplate(m_docManager, _T(""), _T("*.nbf"), _T(""), _T("nbf"), _T("Scene Files"), _T("View"),
      CLASSINFO(VwrDocument), CLASSINFO(VwrView));

  (void) new wxDocTemplate(m_docManager, _T(""), _T("*.wrl"), _T(""), _T("wrl"), _T("Scene Files"), _T("View"),
      CLASSINFO(VwrDocument), CLASSINFO(VwrView));

  m_docManager->SetMaxDocsOpen(1);
    
  // Create the main frame window
  VwrFrame*  frame = new VwrFrame(m_docManager, (wxFrame *) NULL, -1, _T("NVSG Viewer"), wxPoint(0, 0), wxSize(500, 400),   wxDEFAULT_FRAME_STYLE);
    
	frame->Show(TRUE);
	SetTopWindow(frame);
  m_frame = frame;
	return TRUE;
} 

int VwrApp::OnExit(void)
{  
  nvsg::nvsgTerminate();
  return 0;
}

VwrFrame::VwrFrame(wxDocManager *manager, wxFrame *frame, wxWindowID id, const wxString& title, 
   const wxPoint& pos, const wxSize& size,  const long type)
 : wxDocParentFrame(manager, frame, id, title, pos, size, type)
 , m_currentView(NULL)
{
	m_menuFile = new wxMenu;
	
  m_menuFile->Append(wxID_OPEN, _T("&Open..."));
  m_menuFile->Append(wxID_SAVE, _T("&Save..."));
  m_menuFile->Append(wxID_SAVEAS, _T("Save &As..."));
  m_menuFile->Enable(wxID_SAVEAS, true);
  //menuFile->Append( Menu_File_About, wxT( "&About..." ) );	
  m_menuFile->AppendSeparator();
  m_menuFile->Append( Menu_File_Quit, wxT( "E&xit" ) );
  
	 
  //m_docManager->FileHistoryUseMenu(menuFile);

  m_menuNav = new wxMenu;
	
  m_menuNav->Append(MENU_ID_SET_ROTATION_POINT, _T("&Set Rotation Point   (CTRL+LeftMouse)"));
  m_menuNav->Append(MENU_ID_VIEW_ALL, _T("&View All   (CTRL+v)"));  
  m_menuNav->Enable(MENU_ID_SET_ROTATION_POINT, false);
  m_menuNav->Enable(MENU_ID_VIEW_ALL, false);

  wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( m_menuFile, wxT( "&File" ) );
  menuBar->Append( m_menuNav, wxT( "&Navigation" ) );
	
	SetMenuBar( menuBar );
	
	CreateStatusBar();
 	SetStatusText( wxT( "Welcome to NVSG on Linux!" ) );
}

void VwrFrame::OnQuit( wxCommandEvent& WXUNUSED( event ) )
{
 	Close(TRUE);
}

void VwrFrame::OnAbout( wxCommandEvent& WXUNUSED( event ) )
{
	 wxMessageBox( wxT( "NVSG Simple Linux Viewer" ),
            		 wxT( "NVSG" ), wxOK | wxICON_INFORMATION, this );
}

void VwrFrame::OnCloseWindow( wxCloseEvent& event )
{
  // We have to call doc/view cleanup functions here,
  // because OnClose member functions are skipped on EVT_CLOSE.

  VwrView* pView=(VwrView*)m_docManager->GetCurrentView();
  if(pView)
  {
    pView->cleanup();
  }

  VwrDocument* pDoc=(VwrDocument*)m_docManager->GetCurrentDocument();
  if(pDoc)
  {
    pDoc->DeleteContents();
  }  

  Destroy();
}

void VwrFrame::enableMenuNavigation( bool enable=true )
{
  m_menuNav->Enable(MENU_ID_SET_ROTATION_POINT, enable);
  m_menuNav->Enable(MENU_ID_VIEW_ALL, enable);
}

void VwrFrame::OnSetRotationPoint( wxCommandEvent& WXUNUSED( event ) )
{
  if(m_currentView)
  {
    m_currentView->setViewMode(GLwxRAWnd::VM_SET_ROTATION_POINT);
  }  
}

void VwrFrame::OnViewAll( wxCommandEvent& WXUNUSED( event ) )
{
  if(m_currentView)
  {
    m_currentView->viewAll();
  }
}
