#pragma once

#include "nvsg/nvsgapi.h" // NVSG storage-class defines
#include "nvui/RenderArea.h"
#include "nvgl/GLContext.h"

#if defined(LINUX)
# include <X11/Xlib.h>
#endif

namespace nvsg{
  class Scene;
}


/* 
    This class works with GL windows created by the glut library.    
    It will do the necessary initialization and cleanup needed for NVSG, if you
    already have created an OpenGL context through glut, bypassing the NVSG wrapper routines.
    Also it offers the necessary window sizing and mouse event handlers.

    This sample will work on Windows and Linux.
*/

class glutRenderArea :
  public nvui::RenderArea
{
public:
  glutRenderArea(void);
  virtual ~glutRenderArea(void);
  
  // --- The RenderArea interface:

  //! Create the render area.
  virtual bool init( RenderArea * shareArea = NULL  //!< Pointer to a render area to share display lists, textures, vertex programs, etc. with
    );         
  
  //! Cleanup.
  virtual void destroy( void );

  //! Handle the viewport resizing  
  virtual void setViewportSize( size_t width   //!< Width of the viewport
    , size_t height  //!< Height of the viewport
    );

  //! Trigger a redraw.  
  // This will actually never be called as glut will initiate all the repainting through
  // the display routine().
  virtual void triggerRedraw();

  //! Render the scene.
  // This will be called from the glut display() routine.
  virtual void renderScene( void );  

  // --- helper functions:

  inline void makeCurrent();

protected:
  nvsg::Scene*  m_scene;
#if defined(_WIN32)
  HGLRC         m_hglrc;
  HDC           m_hdc;
#else if defined(LINUX)
  GLXDrawable   m_drawable;
  GLXContext    m_hglrc;
  Display*      m_dpy;    
#endif
  
  bool m_initDone;
};

void glutRenderArea::makeCurrent()
{
  // Because of internal nvsg data structures,
  // we have to call our WGLMakeCurrent() wrapper every time, even if glut has already made the context current.  
  // (Also see WGLAttachContext()/WGLDetachContext() call in glutRenderArea.cpp!)
#if defined(_WIN32)
  nvgl::WGLMakeCurrent(m_hdc, m_hglrc);
#else if defined(LINUX)
  nvgl::GLXMakeCurrent(m_dpy, m_drawable, m_hglrc);
#endif
}